/*
 * ServerAppTestTask.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: real_vg, TCSDEVELOPER
 */

using System;
using System.Collections;
using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.NUnit2.Tasks;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Extends NUnit2Task by adding the facility to start/initialize/clean up/shut down
    /// servers related to the unit test cases.
    /// </summary>
    ///
    /// <remarks>
    /// This element may contains one or more "server" child elements (<see cref="ServerElement"/>).
    /// </remarks>
    ///
    /// <example>
    /// Here's a sample NAnt task snippet: <code><![CDATA[
    /// <!-- This NAnt task starts/initializes the servers, runs the tests, and then stops/cleans the servers -->
    /// <serverapptest verbose="true">
    ///
    ///     <!-- This element describes the application server using IIS server -->
    ///     <!-- By default the server is initialized with data (app is deployed) and started -->
    ///     <server class="TopCoder.Test.Framework.Servers.Web.IISServer" initserver="true" startserver="true">
    ///         <options>
    ///             <!-- The name of server service -->
    ///             <option name="service_name" value = "w3svc"/>
    ///             <!-- The directory to deploy application to-->
    ///             <option name="deployment_dir" value = "${deployment_dir}"/>
    ///             <!-- The timeout is set to one minute in milliseconds-->
    ///             <option name="timeout" value = "60000"/>
    ///         </options>
    ///
    ///         <sourcefiles basedir="deploysource">
    ///             <!-- This should be a fileset specifying the files to deploy-->
    ///             <includes name="**"/>
    ///         </sourcefiles>
    ///     </server>
    ///
    ///     <!-- This element describes another sample server -->
    ///     <!-- The server is not started but is initialized with data -->
    ///     <server class="TopCoder.Test.Framework.Servers.Tests.MyServer" startserver="false" initserver="true">
    ///         <options>
    ///             <!-- Specifying the port to connect to -->
    ///             <option name = "port" value = "12345"/>
    ///             <!-- Specifying the address of machine running the server -->
    ///             <option name = "address" value = "topcoder.com"/>
    ///             <!-- Specifying the SQL script used to init the test data -->
    ///             <option name = "sql_init_script" value="script\init.sql"/>
    ///             <!-- Specifying the SQL script used to clean up the test data -->
    ///             <option name = "sql_cleanup_script" value="script\cleanup.sql"/>
    ///         </options>
    ///         <!-- Specifying the credentials used to authenticate to the server -->
    ///         <credentials username="topcoder" password="some_password" />
    ///     </server>
    ///
    ///     <formatter type="Plain"/>
    ///     <test assemblyname="..\build\classes\TopCoder.Test.Framework.Test.Dll"
    ///         testname="TopCoder.Test.Framework.Tests.DemoNUnitAsp" />
    /// </serverapptest>
    /// ]]></code>
    /// </example>
    ///
    /// <threadsafety static="false" instance="false"/>
    [TaskName("serverapptest")]
    public class ServerAppTestTask : NUnit2Task
    {
        /// <summary>
        /// This variable stores the array of server elements.
        /// All elements of the array should be ServerElement instances.
        /// It is accessed through the "Servers" property.
        /// It represents an array of nested "server" XML elements,
        /// describing a particular servers to be started/initialized
        /// while running tests.
        /// This array is used in ExecuteTask() method.
        /// The "serverapptest" NAnt task should include
        /// at least one nested "server" XML element.
        /// Initial value is null.
        /// </summary>
        private ServerElement[] servers;

        /// <summary>
        /// Gets or sets the <see cref="ServerElement"/> array which is an array of
        /// nested "server" Xml elements inside the "serverapptest" task.
        /// </summary>
        ///
        /// <value>
        /// An array of <see cref="ServerElement"/> instances representing
        /// servers that are to be started/initialized while running tests.
        /// This element is required and at least one element should exists.
        /// </value>
        [BuildElementArray("server", ElementType = typeof(ServerElement), Required = true)]
        public ServerElement[] Servers
        {
            get
            {
                return servers;
            }
            set
            {
                servers = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerAppTestTask"/> class.
        /// </summary>
        public ServerAppTestTask()
        {
            // do nothing
        }

        /// <summary>
        /// Executes the task. It first starts all servers, initializes them,
        /// then runs the NUnit test cases, and finally cleans up and shuts down the servers.
        /// </summary>
        ///
        /// <remarks>
        /// If an exception occur while starting/initializing the server, all processes are aborted,
        /// successfully initialized servers are cleaned, and started servers are stopped.
        /// </remarks>
        ///
        /// <exception cref="ServerTaskException">
        /// Any error occurs executing the server task.
        /// The actual exceptions are contained in the inner exceptions array.
        /// </exception>
        /// <exception cref="BuildException">
        /// Any error occurs executing the actual NUnit task.
        /// </exception>
        protected override void ExecuteTask()
        {
            // idx to last servers that are processed successfully
            int idxStarted = -1;
            int idxInitialized = -1;
            // exceptions list
            ArrayList ex = new ArrayList();

            try
            {
                // this 'try' will ensure cleaning/shutting down servers
                try
                {
                    // starts and initializes the servers
                    // one single exception will abort the process
                    StartAndInit(ref idxStarted, ref idxInitialized, ex);

                    // runs the NUnit task (accumulate any exception)
                    try
                    {
                        base.ExecuteTask();
                    }
                    catch (Exception e)
                    {
                        ex.Add(e);
                        throw e;
                    }
                }
                finally
                {
                    // always execute this (even if an exception is thrown from the above process)
                    // cleans up and shuts down servers (all exceptions are accumulated first and rethrown)
                    int beforeCount = ex.Count;
                    CleanUpAndShutdown(idxStarted, idxInitialized, ex);

                    // any exception while cleaning/shutting down?
                    if (ex.Count > beforeCount)
                    {
                        // raise signals for exceptions to be re-caught and re-thrown
                        throw new Exception();
                    }
                }
            }
            catch (Exception)
            {
                // throw ServerTaskException with all exceptions so far as its inner array
                throw new ServerTaskException("Error executing task",
                    (Exception[]) ex.ToArray(typeof(Exception)));
            }
        }

        /// <summary>
        /// Starts servers and initializes servers if all servers can be started successfully.
        /// </summary>
        ///
        /// <param name="idxStarted">
        /// The index to the last server that was started successfully.
        /// </param>
        /// <param name="idxInitialized">
        /// The index to the last server that was initialized successfully.
        /// </param>
        /// <param name="ex">
        /// The IList to add any thrown exception to.
        /// </param>
        ///
        /// <exception cref="ServerException">
        /// When any of the server throws this exception when starting/initializing it.
        /// All starting/initializing sequences are then aborted.
        /// </exception>
        private void StartAndInit(ref int idxStarted, ref int idxInitialized, IList ex)
        {
            // abort process as soon as an exception is detected
            try
            {
                // starts up the servers (increment idxStarted accordingly)
                foreach (ServerElement element in Servers)
                {
                    Server server = element.GetServerInstance();
                    if (element.StartServer)
                    {
                        server.StartUp();
                    }
                    idxStarted++;
                }

                // initializes the servers (increment idxInitialized accordingly)
                foreach (ServerElement element in Servers)
                {
                    Server server = element.GetServerInstance();
                    if (element.InitServer)
                    {
                        server.InitData();
                    }
                    idxInitialized++;
                }
            }
            catch (Exception e)
            {
                // any single exception is added to the IList (to be added later to ServerTaskException)
                // and rethrow
                ex.Add(e);
                throw;
            }
        }

        /// <summary>
        /// Cleans up servers then shuts them down.
        /// </summary>
        ///
        /// <remarks>
        /// Note that if an exception occurs, the exceptions are recorded,
        /// but the cleanup/shutdown process continues to run.
        /// </remarks>
        ///
        /// <param name="idxStarted">
        /// The index to the last server that is to be stopped.
        /// </param>
        /// <param name="idxInitialized">
        /// The index to the last server that is to be cleaned up.
        /// </param>
        /// <param name="ex">
        /// The IList to add any thrown exception to.
        /// </param>
        private void CleanUpAndShutdown(int idxStarted, int idxInitialized, IList ex)
        {
            // cleans up (if an exception occurs, go to next one)
            for (int i = idxInitialized; i >= 0; i--)
            {
                try
                {
                    Server server = Servers[i].GetServerInstance();
                    if (server.InitServer)
                    {
                        server.CleanUpData();
                    }
                }
                catch (Exception e)
                {
                    // collect exception
                    ex.Add(e);
                }
            }

            // shuts down (if an exception occurs, go to next one)
            for (int i = idxStarted; i >= 0; i--)
            {
                try
                {
                    Server server = Servers[i].GetServerInstance();
                    if (server.StartServer)
                    {
                        server.ShutDown();
                    }
                }
                catch (Exception e)
                {
                    // collect exception
                    ex.Add(e);
                }
            }
        }
    }
}
