/*
 * OptionHelper.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: real_vg, TCSDEVELOPER
 */

using System;
using NAnt.Core.Types;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// Several static helper functions for retrieving option value
    /// from a collection of options.
    /// </summary>
    internal sealed class OptionHelper
    {
        /// <summary>
        /// Prevents this class from being instantiated.
        /// </summary>
        private OptionHelper()
        {
        }

        /// <summary>
        /// Gets a string option value for the specified key.
        /// </summary>
        ///
        /// <param name="key">
        /// The key of the option to retrieve.
        /// </param>
        /// <param name="options">
        /// The <see cref="OptionCollection"/> to retrieve from. This could be null.
        /// </param>
        /// <param name="allowNull">
        /// Whether null should be allowed or should cause an exception to be thrown.
        /// </param>
        ///
        /// <returns>
        /// The option value for the specified key or null if option doesn't exist and allowNull is true.
        /// A <see cref="ServerException"/> is thrown if the option doesn't exist and allowNull is false.
        /// </returns>
        ///
        /// <exception cref="ServerException">
        /// The value doesn't exist and allowNull is false.
        /// </exception>
        internal static string GetString(string key, OptionCollection options, bool allowNull)
        {
            // get the option value (required)
            Option opt = ((options == null) ? null : options[key]);
            if ((opt == null) || (opt.Value == null))
            {
                // null is allowed
                if (allowNull)
                {
                    return null;
                }

                // null should throw exception
                throw new ServerException(string.Format(
                    "Missing required '{0}' option", key));
            }
            return opt.Value;
        }

        /// <summary>
        /// Gets and parses an integer option value for the specified key.
        /// </summary>
        ///
        /// <param name="key">
        /// The option key whose value to parse.
        /// </param>
        /// <param name="options">
        /// The <see cref="OptionCollection"/> to retrieve from. This could be null.
        /// </param>
        ///
        /// <returns>
        /// The parsed option value.
        /// </returns>
        ///
        /// <exception cref="ServerException">
        /// The option value doesn't exist or cannot be parsed to an integer value
        /// or is negative.
        /// </exception>
        internal static int GetInt(string key, OptionCollection options)
        {
            // get the string value, throw exception if null
            string optVal = GetString(key, options, false);

            // parse it (cannot be negative)
            try
            {
                int intVal = int.Parse(optVal);
                if (intVal < 0)
                {
                    throw new ServerException(string.Format(
                        "'{0}' value ({1}) cannot be negative", key, intVal));
                }
                return intVal;
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException(string.Format(
                    "Error parsing '{0}' option, value '{1}', to integer", key, optVal), e);
            }
        }

        /// <summary>
        /// Gets and parses an integer option value for the specified key.
        /// A default value is returned if the option doesn't exist.
        /// </summary>
        ///
        /// <param name="key">
        /// The option key whose value to parse.
        /// </param>
        /// <param name="options">
        /// The <see cref="OptionCollection"/> to retrieve from. This could be null.
        /// </param>
        /// <param name="defaultValue">
        /// The value to return if the option doesn't exist.
        /// </param>
        ///
        /// <returns>
        /// The parsed option value or the default value if the option doesn't exist.
        /// </returns>
        ///
        /// <exception cref="ServerException">
        /// If the option value is not null and cannot be parsed to an integer.
        /// </exception>
        internal static int GetInt(string key, OptionCollection options, int defaultValue)
        {
            // get the string value, allow null
            string optVal = GetString(key, options, true);
            if ((optVal == null) || (optVal.Trim().Length == 0))
            {
                // return default value if option doesn't exist
                return defaultValue;
            }

            // parse it (allow negative)
            try
            {
                return int.Parse(optVal);
            }
            catch (Exception e)
            {
                throw new ServerException(string.Format(
                    "Error parsing '{0}' option, value '{1}', to integer", key, optVal), e);
            }
        }
    }
}
