/*
 * IISServer.cs
 *
 * Testing Framework
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: real_vg, TCSDEVELOPER
 */

using System;
using System.ServiceProcess;
using NAnt.Core.Tasks;

namespace TopCoder.Test.Framework.Servers.Web
{
    /// <summary>
    /// Represents a web application server to be used while running tests.
    /// </summary>
    ///
    /// <remarks>
    /// <para>
    /// It currently supports the Microsoft IIS server, although other service-based servers
    /// could be handled as well.
    /// The following options are supported:
    /// <list type="table">
    /// <listheader>
    ///     <term>Option name</term><description>Description</description>
    /// </listheader>
    /// <item>
    ///     <term><b><see cref="SERVICE_NAME"/></b></term>
    ///     <description>
    ///     The service name of the server.
    ///     This option is optional and has a default of "w3svc" to represent IIS.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term><b><see cref="ApplicationServer.DEPLOYMENT_DIR"/></b></term>
    ///     <description>
    ///         This option comes from the <see cref="ApplicationServer"/>,
    ///         it is required when the "initserver" attribute is true.
    ///     </description>
    /// </item>
    /// <item>
    ///     <term><b><see cref="Server.TIMEOUT"/></b></term>
    ///     <description>
    ///         This option comes from the <see cref="Server"/>, it is optional (default is 1 minute).
    ///         It defines the time to wait for service to read the needed state.
    ///     </description>
    /// </item>
    /// </list>
    /// The NUnitAsp can be used for writing tests for IIS server
    /// when using this component.
    /// </para>
    /// <para>
    /// <b>Warning</b>: Note that <see cref="StartUp"/> will deploy files specified by the sourcefiles tag.
    /// These sourcefiles pattern are also used to delete files from the deployment directory
    /// in <see cref="ShutDown"/>. Be careful about using * / ** wildcard to match everything
    /// because this will also remove all files/directories that may have existed in the deployment directory.
    /// Consider using <b>excludes</b> tag to exclude those files from being removed.
    /// </para>
    /// <para>
    /// Also note that the deployment directory will be automatically created if it doesn't exist yet,
    /// however depending on the sourcefiles pattern, they may not be removed on <see cref="ShutDown"/>.
    /// For example, the pattern ** will remove the deployment directory,
    /// however the pattern * will only remove the files.
    /// And the pattern *\** will not remove the deployment directory itself.
    /// </para>
    /// </remarks>
    ///
    /// <example>
    /// See <see cref="Server"/> for example NAnt snippet.
    /// </example>
    ///
    /// <threadsafety static="false" instance="false"/>
    public class IISServer : ApplicationServer
    {
        /// <summary>
        /// A string constant that represents the name (key) of the option
        /// for getting the service name of the server.
        /// </summary>
        ///
        /// <remarks>
        /// This option is not required and has a default of "w3svc" to represent IIS.
        /// </remarks>
        public static readonly string SERVICE_NAME = "service_name";

        /// <summary>
        /// Represents a default value for the SERVICE_NAME option.
        /// It is the name of IIS service.
        /// </summary>
        private static readonly string DEFAULT_SERVICE_NAME = "w3svc";

        /// <summary>
        /// Represents default value for the TIMEOUT option.
        /// The default value is a time of 1 minute, represented in milliseconds.
        /// </summary>
        private static readonly int DEFAULT_TIMEOUT = 60000;

        /// <summary>
        /// Initializes a new instance of the <see cref="IISServer"/> class.
        /// </summary>
        public IISServer()
        {
            // do nothing
        }

        /// <summary>
        /// Deploys and starts the server service.
        /// </summary>
        ///
        /// <remarks>
        /// If <see cref="Server.InitServer"/> is true, the application/sourcefiles
        /// will be copied to the deployment directory.
        /// And then the server service is started.
        /// If the service couldn't be started in the time specified by <see cref="Server.Timeout"/> option,
        /// an exception is thrown.
        /// </remarks>
        ///
        /// <exception cref="ServerException">
        /// Any error happens while deploying or starting the server.
        /// </exception>
        public override void StartUp()
        {
            try
            {
                // initialize the server if requested
                if (InitServer)
                {
                    // copy all SourceFiles to DeploymentDirectory (overwrite existing files)
                    CopyTask copyTask = new CopyTask();
                    copyTask.Project = Project;
                    copyTask.ToDirectory = DeploymentDirectory;
                    copyTask.CopyFileSet = SourceFiles;
                    copyTask.Overwrite = true;
                    copyTask.Execute();
                }

                // start or resume the service
                ServiceHandler(ServiceControllerStatus.Running);
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException("Error starting up/initializing service", e);
            }
        }

        /// <summary>
        /// Stops and cleans up the server.
        /// </summary>
        ///
        /// <remarks>
        /// First the server service is stopped.
        /// If <see cref="Server.InitServer"/> is true, the application/sourcefiles
        /// will be deleted from the deployment directory.
        /// If the service couldn't be stopped in the time specified by <see cref="Server.Timeout"/> option,
        /// an exception is thrown.
        /// </remarks>
        ///
        /// <exception cref="ServerException">
        /// Any error happens while stopping or cleaning up the server.
        /// </exception>
        public override void ShutDown()
        {
            // since we're going to alter SourceFiles.BaseDirectory
            // make sure we restore it later
            string originalBase = SourceFiles.BaseDirectory;

            try
            {
                // stop the service
                ServiceHandler(ServiceControllerStatus.Stopped);

                // cleans up the server if requested
                if (InitServer)
                {
                    // delete all SourceFiles from DeploymentDirectory
                    DeleteTask deleteTask = new DeleteTask();
                    deleteTask.DeleteFileSet = SourceFiles;
                    deleteTask.DeleteFileSet.BaseDirectory = Project.GetFullPath(DeploymentDirectory);
                    deleteTask.DeleteFileSet.Reset();   // force a re-scan in the new BaseDirectory
                    deleteTask.Project = Project;
                    deleteTask.Execute();
                }
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException("Error stopping/cleaning up service", e);
            }
            finally
            {
                // restore BaseDirectory of SourceFiles
                SourceFiles.BaseDirectory = originalBase;
                SourceFiles.Reset();
            }
        }

        /// <summary>
        /// This implementation does nothing. Application is deployed in <see cref="StartUp"/>.
        /// </summary>
        public override void InitData()
        {
            // do nothing
        }

        /// <summary>
        /// This implementation does nothing. Application is cleaned-up in <see cref="ShutDown"/>.
        /// </summary>
        public override void CleanUpData()
        {
            // do nothing
        }

        /// <summary>
        /// Starts or stops a service.
        /// If expected equals to ServiceControllerStatus.Running, the service
        /// will be resumed/started (current status should be paused/stopped).
        /// If expected equals to ServiceControllerStatus.Stopped, the service
        /// will be stopped (current status should be running).
        /// </summary>
        ///
        /// <param name="expected">
        /// The expected status of the service. Only ServiceControllerStatus.Running
        /// and ServiceControllerStatus.Stopped is valid here.
        /// </param>
        ///
        /// <exception cref="ServerException">
        /// The current status is not what is expected or timeout waiting for the service
        /// to reach the expected state.
        /// </exception>
        private void ServiceHandler(ServiceControllerStatus expected)
        {
            try
            {
                // get service name (use DEFAULT_SERVICE_NAME is not specified)
                string serviceName = OptionHelper.GetString(SERVICE_NAME, Options, true);
                if ((serviceName == null) || (serviceName.Trim().Length == 0))
                {
                    serviceName = DEFAULT_SERVICE_NAME;
                }

                // get timeout value (use DEFAULT_TIMEOUT if not specified)
                int timeout = OptionHelper.GetInt(TIMEOUT, Options, DEFAULT_TIMEOUT);

                ServiceController service = new ServiceController(serviceName);
                if (expected == ServiceControllerStatus.Stopped)
                {
                    // stop the service
                    if (service.CanStop)
                    {
                        service.Stop();
                    }
                    else
                    {
                        throw new ServerException(string.Format(
                            "Service {0} is in unexpected state '{1}' and cannot be stopped",
                            serviceName, service.Status.ToString()));
                    }
                }
                else
                {
                    // start the service
                    if (service.Status == ServiceControllerStatus.Paused)
                    {
                        service.Continue();
                    }
                    else if (service.Status == ServiceControllerStatus.Stopped)
                    {
                        service.Start();
                    }
                    else
                    {
                        throw new ServerException(string.Format(
                            "Service {0} is in unexpected state '{1}' and cannot be started",
                            serviceName, service.Status.ToString()));
                    }
                }

                // wait until timeout for the service to run/stop
                service.WaitForStatus(expected, TimeSpan.FromMilliseconds(timeout));
            }
            catch (ServerException)
            {
                throw;
            }
            catch (TimeoutException e)
            {
                throw new ServerException(string.Format(
                    "Timeout waiting for the service to {0}", expected.ToString()), e);
            }
            catch (Exception e)
            {
                throw new ServerException("Error stopping/cleaning up service", e);
            }
        }
    }
}
